@@ -48,7 +48,7 @@ FrameExporter::FrameExporter() {
     cond   = SDL_CreateCond();
     mutex  = SDL_CreateMutex();
 
-#if SDL_VERSION_ATLEAST(1,3,0)
+#if SDL_VERSION_ATLEAST(2,0,0)
     thread = SDL_CreateThread( dumper_thread, "frame_exporter", this );
 #else
     thread = SDL_CreateThread( dumper_thread, this );
@@ -58,7 +58,6 @@ FrameExporter::FrameExporter() {
 FrameExporter::~FrameExporter() {
     stop();
 
-    SDL_KillThread(thread);
     SDL_DestroyCond(cond);
     SDL_DestroyMutex(mutex);
 
@@ -70,6 +69,8 @@ FrameExporter::~FrameExporter() {
 }
 
 void FrameExporter::stop() {
+    if(!thread) return;
+    
     if(dumper_thread_state == FRAME_EXPORTER_STOPPED || dumper_thread_state == FRAME_EXPORTER_EXIT) return;
 
     SDL_mutexP(mutex);
@@ -80,9 +81,9 @@ void FrameExporter::stop() {
 
     SDL_mutexV(mutex);
 
-    //busy wait for thread to exit
-    while(dumper_thread_state != FRAME_EXPORTER_STOPPED)
-        SDL_Delay(100);
+    SDL_WaitThread(thread, 0);
+    
+    thread = 0;
 }
 
 void FrameExporter::dump() {
