@@ -16,6 +16,7 @@
 */
 
 #include "gource.h"
+#include "core/png_writer.h"
 
 bool  gGourceDrawBackground  = true;
 bool  gGourceQuadTreeDebug   = false;
@@ -97,6 +98,8 @@ Gource::Gource(FrameExporter* exporter) {
     mousedragged = false;
     mouseclicked = false;
 
+    take_screenshot = false;
+
     if(gGourceSettings.hide_mouse) {
         cursor.showCursor(false);
     }
@@ -141,8 +144,6 @@ Gource::Gource(FrameExporter* exporter) {
 
     logmill = new RLogMill(logfile);
 
-    shutdown = false;
-
     if(exporter!=0) setFrameExporter(exporter, gGourceSettings.output_framerate);
 
     //if recording a video or in demo mode, or multiple repos, the slider is initially hidden
@@ -222,7 +223,6 @@ void Gource::reload() {
 }
 
 void Gource::quit() {
-    shutdown = true;
 }
 
 void Gource::update(float t, float dt) {
@@ -251,7 +251,7 @@ void Gource::update(float t, float dt) {
     draw(runtime, scaled_dt);
 
     //extract frames based on frameskip setting if frameExporter defined
-    if(frameExporter != 0 && commitlog && !shutdown) {
+    if(frameExporter != 0 && commitlog && !gGourceSettings.shutdown) {
         if(framecount % (frameskip+1) == 0) {
             frameExporter->dump();
         }
@@ -286,14 +286,42 @@ std::string Gource::dateAtPosition(float percent) {
     return date;
 }
 
+void Gource::grabMouse(bool grab_mouse) {
+    this->grab_mouse = grab_mouse;
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+    if(grab_mouse) {
+        if(!SDL_GetRelativeMouseMode()) {
+            // NOTE: SDL_SetWindowGrab needed as well to work around this bug:
+            // http://bugzilla.libsdl.org/show_bug.cgi?id=1967
+
+            SDL_SetWindowGrab(display.sdl_window, SDL_TRUE);
+            SDL_SetRelativeMouseMode(SDL_TRUE);
+        }
+    } else {
+        if(SDL_GetRelativeMouseMode()) {
+            SDL_SetWindowGrab(display.sdl_window, SDL_FALSE);
+            SDL_SetRelativeMouseMode(SDL_FALSE);
+        }
+    }
+#endif
+
+#if not SDL_VERSION_ATLEAST(2,0,0)
+    if(!grab_mouse) {
+        // restore old mouse position
+        SDL_WarpMouse(mousepos.x, mousepos.y);
+    }
+#endif
+
+    cursor.showCursor(!grab_mouse);
+}
+
 void Gource::mouseMove(SDL_MouseMotionEvent *e) {
     if(commitlog==0) return;
     if(gGourceSettings.hide_mouse) return;
 
-    bool rightmouse = cursor.rightButtonPressed();
-
-#if not SDL_VERSION_ATLEAST(1,3,0)
     if(grab_mouse) {
+#if not SDL_VERSION_ATLEAST(2,0,0)
          int warp_x = display.width/2;
          int warp_y = display.height/2;
 
@@ -301,8 +329,10 @@ void Gource::mouseMove(SDL_MouseMotionEvent *e) {
          if(e->x == warp_x && e->y == warp_y) return;
 
          SDL_WarpMouse(warp_x, warp_y);
-    }
 #endif
+    }
+
+    bool rightmouse = cursor.rightButtonPressed();
 
     //move camera in direction the user dragged the mouse
     if(mousedragged || rightmouse) {
@@ -366,14 +396,9 @@ void Gource::zoom(bool zoomin) {
     camera.setDistance(distance);
 }
 
-#if SDL_VERSION_ATLEAST(1,3,0)
+#if SDL_VERSION_ATLEAST(2,0,0)
 void Gource::mouseWheel(SDL_MouseWheelEvent *e) {
 
-   int mouse_x, mouse_y;
-    SDL_GetMouseState(&mouse_x, &mouse_y);
-
-    vec2 mousepos(mouse_x, mouse_y);
-
     if(e->y > 0) {
         zoom(true);
     }
@@ -402,20 +427,15 @@ void Gource::mouseClick(SDL_MouseButtonEvent *e) {
 
         if(e->button == SDL_BUTTON_LEFT || e->button == SDL_BUTTON_RIGHT) {
             if(!cursor.buttonPressed()) {
-                grab_mouse=false;
-#if SDL_VERSION_ATLEAST(1,3,0)
-		SDL_SetRelativeMouseMode(SDL_FALSE);
-		SDL_WarpMouseInWindow(display.sdl_window, mousepos.x, mousepos.y);
-#else
-		SDL_WarpMouse(mousepos.x, mousepos.y);
-#endif
-                cursor.showCursor(true);
+                grabMouse(false);
             }
         }
     }
 
     if(e->type != SDL_MOUSEBUTTONDOWN) return;
 
+
+#if not SDL_VERSION_ATLEAST(2,0,0)
     //wheel up
     if(e->button == SDL_BUTTON_WHEELUP) {
         zoom(true);
@@ -427,6 +447,7 @@ void Gource::mouseClick(SDL_MouseButtonEvent *e) {
         zoom(false);
         return;
     }
+#endif
 
     if(e->button == SDL_BUTTON_MIDDLE) {
         toggleCameraMode();
@@ -435,11 +456,7 @@ void Gource::mouseClick(SDL_MouseButtonEvent *e) {
 
 
     if(e->button == SDL_BUTTON_RIGHT) {
-        cursor.showCursor(false);
-        grab_mouse=true;
-#if SDL_VERSION_ATLEAST(1,3,0)
-        SDL_SetRelativeMouseMode(SDL_TRUE);
-#endif
+        grabMouse(true);
         return;
     }
 
@@ -510,9 +527,9 @@ void Gource::selectBackground() {
 
     manual_camera = true;
 
-    cursor.showCursor(false);
-    grab_mouse=true;
     mousedragged=true;
+
+    grabMouse(true);
 }
 
 //select a user, deselect current file/user
@@ -616,18 +633,19 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
 
     if (e->type == SDL_KEYDOWN) {
 
-#if SDL_VERSION_ATLEAST(1,3,0)
-        bool key_escape       = e->keysym.scancode == SDL_SCANCODE_ESCAPE;
-        bool key_tab          = e->keysym.scancode == SDL_SCANCODE_TAB;
-        bool key_space        = e->keysym.scancode == SDL_SCANCODE_SPACE;
-        bool key_plus         = e->keysym.scancode == SDL_SCANCODE_EQUALS;
-        bool key_equals       = e->keysym.scancode == SDL_SCANCODE_EQUALS;
-        bool key_minus        = e->keysym.scancode == SDL_SCANCODE_MINUS;
-        bool key_leftbracket  = e->keysym.scancode == SDL_SCANCODE_LEFTBRACKET;
-        bool key_rightbracket = e->keysym.scancode == SDL_SCANCODE_RIGHTBRACKET;
-        bool key_comma        = e->keysym.scancode == SDL_SCANCODE_COMMA;
-        bool key_period       = e->keysym.scancode == SDL_SCANCODE_PERIOD;
-        bool key_slash        = e->keysym.scancode == SDL_SCANCODE_SLASH;
+
+#if SDL_VERSION_ATLEAST(2,0,0)
+        bool key_escape       = e->keysym.sym == SDLK_ESCAPE;
+        bool key_tab          = e->keysym.sym == SDLK_TAB;
+        bool key_space        = e->keysym.sym == SDLK_SPACE;
+        bool key_plus         = e->keysym.sym == SDLK_PLUS;
+        bool key_equals       = e->keysym.sym == SDLK_EQUALS;
+        bool key_minus        = e->keysym.sym == SDLK_MINUS;
+        bool key_leftbracket  = e->keysym.sym == SDLK_LEFTBRACKET;
+        bool key_rightbracket = e->keysym.sym == SDLK_RIGHTBRACKET;
+        bool key_comma        = e->keysym.sym == SDLK_COMMA;
+        bool key_period       = e->keysym.sym == SDLK_PERIOD;
+        bool key_slash        = e->keysym.sym == SDLK_SLASH;
 #else
         bool key_escape       = e->keysym.unicode == SDLK_ESCAPE;
         bool key_tab          = e->keysym.unicode == SDLK_TAB;
@@ -641,7 +659,6 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
         bool key_period       = e->keysym.unicode == SDLK_PERIOD;
         bool key_slash        = e->keysym.unicode == SDLK_SLASH;
 #endif
-
         if (key_escape) {
             quit();
         }
@@ -649,7 +666,7 @@ void Gource::keyPress(SDL_KeyboardEvent *e) {
         if(commitlog==0) return;
 
         if(e->keysym.sym == SDLK_F12) {
-            screenshot();
+            take_screenshot = true;
         }
 
         if (e->keysym.sym == SDLK_q) {
@@ -1086,7 +1103,7 @@ void Gource::readLog() {
     //debugLog("readLog()\n");
 
     // read commits until either we are ahead of currtime
-    while(!commitlog->isFinished() && (commitqueue.empty() || (commitqueue.back().timestamp <= currtime && commitqueue.size() < commitqueue_max_size)) ) {
+    while((commitlog->hasBufferedCommit() || !commitlog->isFinished()) && (commitqueue.empty() || (commitqueue.back().timestamp <= currtime && commitqueue.size() < commitqueue_max_size)) ) {
 
         RCommit commit;
 
@@ -1097,7 +1114,12 @@ void Gource::readLog() {
             continue;
         }
 
-         commitqueue.push_back(commit);
+        if(gGourceSettings.stop_timestamp != 0 && commit.timestamp > gGourceSettings.stop_timestamp) {
+            stop_position_reached = true;
+            break;
+        }
+
+        commitqueue.push_back(commit);
     }
 
     if(first_read && commitqueue.empty()) {
@@ -1113,8 +1135,14 @@ void Gource::readLog() {
 
     bool is_finished = commitlog->isFinished();
 
-    if(   (gGourceSettings.stop_at_end && is_finished)
-       || (gGourceSettings.stop_position > 0.0 && commitlog->isSeekable() && (is_finished || last_percent >= gGourceSettings.stop_position)) ) {
+
+    if(
+       // end reached
+       (gGourceSettings.stop_at_end && is_finished)
+
+       // stop position reached
+       || (gGourceSettings.stop_position > 0.0 && commitlog->isSeekable() && (is_finished || last_percent >= gGourceSettings.stop_position))
+    ) {
         stop_position_reached = true;
     }
 
@@ -1504,7 +1532,7 @@ void Gource::changeColours() {
 
 void Gource::logic(float t, float dt) {
 
-    if(shutdown && logmill->isFinished()) {
+    if(gGourceSettings.shutdown && logmill->isFinished()) {
         appFinished=true;
         return;
     }
@@ -1552,8 +1580,8 @@ void Gource::logic(float t, float dt) {
     bool up    = false;
     bool down  = false;
 
-#if SDL_VERSION_ATLEAST(1,3,0)
-    Uint8 *keystate = SDL_GetKeyboardState(0);
+#if SDL_VERSION_ATLEAST(2,0,0)
+    const Uint8 *keystate = SDL_GetKeyboardState(0);
 
     right = keystate[SDL_SCANCODE_RIGHT];
     left  = keystate[SDL_SCANCODE_LEFT];
@@ -1931,7 +1959,7 @@ void Gource::loadingScreen() {
             break;
     }
 
-    const char* action = !shutdown ? "Reading Log" : "Aborting";
+    const char* action = !gGourceSettings.shutdown ? "Reading Log" : "Aborting";
 
     int width = font.getWidth(action);
     font.setColour(vec4(1.0f));
@@ -2150,23 +2178,23 @@ void Gource::setMessage(const char* str, ...) {
 void Gource::screenshot() {
 
     //get next free recording name
-    char tganame[256];
+    char pngname[256];
     struct stat finfo;
-    int tgano = 1;
+    int pngno = 1;
 
-    while(tgano < 10000) {
-        snprintf(tganame, 256, "gource-%04d.tga", tgano);
-        if(stat(tganame, &finfo) != 0) break;
-        tgano++;
+    while(pngno < 10000) {
+        snprintf(pngname, 256, "gource-%04d.png", pngno);
+        if(stat(pngname, &finfo) != 0) break;
+        pngno++;
     }
 
-    //write tga
-    std::string filename(tganame);
+    //write png
+    std::string filename(pngname);
 
-    TGAWriter tga(gGourceSettings.transparent ? 4 : 3);
-    tga.screenshot(filename);
+    PNGWriter png(gGourceSettings.transparent ? 4 : 3);
+    png.screenshot(filename);
 
-    setMessage("Wrote screenshot %s", tganame);
+    setMessage("Wrote screenshot %s", pngname);
 }
 
 void Gource::updateVBOs(float dt) {
@@ -2573,10 +2601,6 @@ void Gource::draw(float t, float dt) {
         caption->draw();
     }
 
-    if(message_timer>0.0f) {
-         fontmedium.draw(1, 3, message);
-    }
-
     //file key
     file_key.draw();
     file_key.setShow(gGourceSettings.show_key);
@@ -2674,4 +2698,19 @@ void Gource::draw(float t, float dt) {
 
     mousemoved=false;
     mouseclicked=false;
+
+    if(take_screenshot) {
+        screenshot();
+        take_screenshot = false;
+    }
+
+    if(message_timer > 0.0f) {
+        font.setColour(vec4(1.0f));
+
+        glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+        glEnable(GL_BLEND);
+        glEnable(GL_TEXTURE_2D);
+
+        font.draw(1, 3, message);
+    }
 }
